/*************************************************************************
 *
 *    Used with ICCARM and AARM.
 *
 *    (c) Copyright IAR Systems 2005
 *
 *    File name   : drv_hd44780_l.c
 *    Description : I/O function
 *
 *    History :
 *    1. Data        : June 13, 2005
 *       Author      : Stanimir Bonev
 *       Description : Create
 *
 *    $Revision: 1.1 $
**************************************************************************/

void HD44780_IO_Init (void);
void HD44780SetRS (Boolean Data);
void HD44780SetE (Boolean Data);
#if HD4780_WR > 0
void HD44780SetRW (Boolean Data);
Int8U HD44780RdIO (void);
#endif
void HD44780WrIO (Int8U);

/*************************************************************************
 * Function Name: HD44780_IO_Init
 * Parameters: none
 * Return: none
 * Description: Init IO ports directions and level
 *
 *************************************************************************/
void HD44780_IO_Init (void)
{

  // Init DATA IO P8
//  SCU_PCGR1_bit.GPIO8   = 1;        // GPIO8 Port clock runing
  SCU_PRR1_bit.RST_GPIO8= 1;        // GPIO8 out of reset
  SCU_EMI_bit.GPIOEMI   = 0;        // P8, P9 general IO
  SCU_GPIOTYPE8         = 0;        // GPIO3 Push-Pull outputs
  GPIO8_SEL             = 0;
  GPIO8_DIR             = 0x00;
  GPIO8_DATA            = 0xFF;

  // Init P9 LCD Ctrl
//  SCU_PCGR1_bit.GPIO9   = 1;        // GPIO9 Port clock runing
  SCU_PRR1_bit.RST_GPIO9= 1;        // GPIO9 out of reset
  SCU_GPIOTYPE9         = 0;        // GPIO9 Push-Pull outputs
  GPIO9_SEL             = 0;
  GPIO9_DIR            |= LCD_RS | LCD_RW | LCD_E | LCD_LIGHT;

  // RS Ouput - Low
  HD44780SetRS(0);
  // E  Ouput - Low
  HD44780SetE(0);
#if HD4780_WR > 0
  // WR Ouput - Low
  HD44780SetRW(0);
#endif
}

/*************************************************************************
 * Function Name: HD44780SetRS
 * Parameters: Boolean Data
 * Return: none
 * Description: Set RS signal
 *
 *************************************************************************/
void HD44780SetRS (Boolean Data)
{
  if(Data)
  {
    GPIO9_DATA |= LCD_RS;
  }
  else
  {
    GPIO9_DATA &= ~LCD_RS;
  }
}

/*************************************************************************
 * Function Name: HD44780SetE
 * Parameters: Boolean Data
 * Return: none
 * Description: Set E signal
 *
 *************************************************************************/
void HD44780SetE (Boolean Data)
{
  if(Data)
  {
    GPIO9_DATA |= LCD_E;
  }
  else
  {
    GPIO9_DATA &= ~LCD_E;
  }
}

/*************************************************************************
 * Function Name: HD44780WrIO
 * Parameters: Int8U Data
 * Return: none
 * Description: Write to HD44780 I/O
 *
 *************************************************************************/
void HD44780WrIO (Int8U Data)
{
  // Set Direction
  GPIO8_DIR  = 0xFF;
#if HD4780_WR > 0
  HD44780SetRW(0);
#endif
  // Write Data
  // Clock E
  HD44780SetE(1);

  GPIO8_DATA = Data;
  HD44780_BUS_DLY();
  HD44780SetE(0);
}

#if HD4780_WR > 0
/*************************************************************************
 * Function Name: HD44780SetRW
 * Parameters: Boolean Data
 * Return: none
 * Description: Set R/W signal
 *
 *************************************************************************/
void HD44780SetRW (Boolean Data)
{
  if(Data)
  {
    GPIO9_DATA |= LCD_RW;
  }
  else
  {
    GPIO9_DATA &= ~LCD_RW;
  }
}

/*************************************************************************
 * Function Name: HD44780RdIO
 * Parameters: none
 * Return: Int8U
 * Description: Read from HD44780 I/O
 *
 *************************************************************************/
Int8U HD44780RdIO (void)
{
Int8U Data;
  // Set Direction
  GPIO8_DIR  = 0x00;
  HD44780SetRW(1);
  // Read Data
  HD44780SetE(1);
  HD44780_BUS_DLY();
  Data = GPIO8_DATA;
  HD44780SetE(0);
  // Clock E
  return Data;
}
#endif
